Package: hardhat
Title: Construct Modeling Packages
Version: 1.4.3
Authors@R: c(
    person("Hannah", "Frick", , "hannah@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-6049-5258")),
    person("Davis", "Vaughan", , "davis@posit.co", role = "aut"),
    person("Max", "Kuhn", , "max@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Building modeling packages is hard. A large amount of effort
    generally goes into providing an implementation for a new method that
    is efficient, fast, and correct, but often less emphasis is put on the
    user interface. A good interface requires specialized knowledge about
    S3 methods and formulas, which the average package developer might not
    have. The goal of 'hardhat' is to reduce the burden around building
    new modeling packages by providing functionality for preprocessing,
    predicting, and validating input.
License: MIT + file LICENSE
URL: https://github.com/tidymodels/hardhat,
        https://hardhat.tidymodels.org
BugReports: https://github.com/tidymodels/hardhat/issues
Depends: R (>= 4.1)
Imports: cli (>= 3.6.0), glue (>= 1.6.2), rlang (>= 1.1.0), sparsevctrs
        (>= 0.2.0), tibble (>= 3.2.1), vctrs (>= 0.6.0)
Suggests: covr, crayon, devtools, knitr, Matrix, modeldata (>= 0.0.2),
        recipes (>= 1.3.0), rmarkdown (>= 2.3), roxygen2, testthat (>=
        3.0.0), usethis (>= 2.1.5), withr (>= 3.0.0)
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-23
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-02 19:07:16 UTC; hannah
Author: Hannah Frick [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6049-5258>),
  Davis Vaughan [aut],
  Max Kuhn [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Hannah Frick <hannah@posit.co>
Repository: CRAN
Date/Publication: 2026-04-04 16:50:02 UTC
Built: R 4.5.2; ; 2026-04-04 19:24:03 UTC; unix
