#' Example data for hardhat
#'
#' @details Data objects for a training and test set with the same variables:
#' three numeric and two factor columns.
#'
#' @name hardhat-example-data
#' @aliases example_train example_test
#' @docType data
#' @return \item{example_train,example_test}{tibbles}
#'
#' @keywords datasets
#' @examples
#' data("hardhat-example-data")
NULL
