#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import rlang
#' @import vctrs
#' @importFrom glue glue
#' @importFrom tibble tibble
#' @importFrom stats model.frame
#' @importFrom stats model.matrix
#' @importFrom stats delete.response
#' @importFrom stats get_all_vars
#' @importFrom stats terms
## usethis namespace: end
NULL
