% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/case-weights.R
\name{is_frequency_weights}
\alias{is_frequency_weights}
\title{Is \code{x} a frequency weights vector?}
\usage{
is_frequency_weights(x)
}
\arguments{
\item{x}{An object.}
}
\value{
A single \code{TRUE} or \code{FALSE}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{is_frequency_weights()} checks if \code{x} inherits from
\code{"hardhat_frequency_weights"}.
}
\examples{
is_frequency_weights(1)
is_frequency_weights(frequency_weights(1))
is_frequency_weights(importance_weights(1))
}
