% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/case-weights.R
\name{frequency_weights}
\alias{frequency_weights}
\title{Frequency weights}
\usage{
frequency_weights(x)
}
\arguments{
\item{x}{An integer vector.}
}
\value{
A new frequency weights vector.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{frequency_weights()} creates a vector of frequency weights which allow you
to compactly repeat an observation a set number of times. Frequency weights
are supplied as a non-negative integer vector, where only whole numbers are
allowed.
}
\examples{
# Record that the first observation has 10 replicates, the second has 12
# replicates, and so on
frequency_weights(c(10, 12, 2, 1))

# Fractional values are not allowed
try(frequency_weights(c(1.5, 2.3, 10)))
}
\seealso{
\code{\link[=importance_weights]{importance_weights()}}
}
